import requests
import zipfile
import os
import tkinter as tk
from tkinter import ttk
from tkinter import messagebox


class UpdateChecker:
    def __init__(self, project, repo, branch):
        self.project = project
        self.repo = repo
        self.branch = branch
        self.version_url = f"https://raw.githubusercontent.com/{project}/{branch}/VERSION"
        self.archive_url = f"https://github.com/{project}/archive/refs/heads/{branch}.zip"

    def get_current_version(self):
        try:
            response = requests.get(self.version_url)
            response.raise_for_status()
            return response.text.strip()
        except Exception as e:
            print(f"Error fetching version: {e}")
            return None

    def download_and_extract(self):
        download_path = "temp.zip"
        extract_path = "."  # Extract directly into the current folder (same as the updater)

        # Start the GUI window for downloading and extracting
        self.show_progress_bar()

        try:
            # Download the zip file
            response = requests.get(self.archive_url, stream=True)
            response.raise_for_status()
            with open(download_path, "wb") as file:
                for chunk in response.iter_content(chunk_size=10 * 1024):
                    file.write(chunk)

            # Extract the zip file
            with zipfile.ZipFile(download_path, "r") as zip_ref:
                # Find the folder inside the zip file (assuming there is only one top-level folder)
                folder_name = zip_ref.namelist()[0].split('/')[0]  # Get the first folder name

                for file_name in zip_ref.namelist():
                    # Skip if the file is part of a subfolder
                    if file_name.startswith(folder_name):
                        # Get the relative file path by removing the folder name prefix
                        relative_path = file_name[len(folder_name) + 1:]
                        target_path = os.path.join(extract_path, relative_path)

                        os.makedirs(os.path.dirname(target_path), exist_ok=True)

                        if not file_name.endswith("/"):  # Ignore directories
                            with open(target_path, "wb") as target_file:
                                target_file.write(zip_ref.read(file_name))
                            print(f"Extracted: {file_name} to {target_path}")

            # Clean up the downloaded zip file
            os.remove(download_path)

            # Stop the progress bar and show a success message
            self.hide_progress_bar()
            messagebox.showinfo("Success", "Update downloaded and extracted successfully!")

        except Exception as e:
            # Stop the progress bar and show an error message
            self.hide_progress_bar()
            messagebox.showerror("Error", f"Error downloading or extracting files: {e}")

    def check_for_updates(self):
        current_version = self.get_current_version()
        if not current_version:
            print("Failed to fetch the current version.")
            return

        try:
            with open("VERSION", "r") as version_file:
                base_version = version_file.read().strip()
        except FileNotFoundError:
            print("Local VERSION file not found.")
            return

        if current_version > base_version:
            print("New update available. Downloading and extracting...")
            self.download_and_extract()
        elif current_version < base_version:
            print("Files are corrupted! Restoring...")
            self.download_and_extract()
        else:
            print("No new updates detected.")

    def show_progress_bar(self):
        # Create a simple tkinter window for the GUI
        self.window = tk.Tk()
        self.window.title("Update Progress")
        self.window.geometry("400x150")

        self.progress_label = tk.Label(self.window, text="Downloading and extracting update...")
        self.progress_label.pack(pady=10)

        self.progress_bar = ttk.Progressbar(self.window, orient="horizontal", length=300, mode="indeterminate")
        self.progress_bar.pack(pady=10)
        self.progress_bar.start()

    def hide_progress_bar(self):
        if self.window:
            self.window.quit()
            self.window.destroy()

    def run(self):
        self.check_for_updates()

